#!/bin/sh

# version of this shell script.
OS_KIND=3
MAJOR_VER=4
RELEASE_VER=8.0
VERSION=${OS_KIND}.${MAJOR_VER}.${RELEASE_VER}

# snvcli package info
CLI_MAJOR_VER=4
CLI_RELEASE_VER=8.0
CLI_VERSION=${OS_KIND}.${CLI_MAJOR_VER}-${CLI_RELEASE_VER}

# hrn package info
HRN_MAJOR_VER=3.16
HRN_RELEASE_VER=0.0
HRN_VERSION=${HRN_MAJOR_VER}-${HRN_RELEASE_VER}


CLI_PACKAGE_NAME=snvcli
HRN_PACKAGE_NAME=hrn
CMDNAME=`basename $0`
RPMPATH=`dirname $0`
SNVPATH=/opt/hitachi/snv
LOGDIR=/var/log/snv
LOGFILE=$LOGDIR/snv_setup.log
BACKUP_TEMP=/opt/hitachi/snvtmp
TMPPROCS=$LOGDIR/snvprocs.$$
EXEC_COUNT=0

ARCH=
uname -m | grep -q "x86_64"
if [ $? -eq 0 ] ; then
	ARCH="x86_64"
else
	ARCH="i386"
fi

SNV_SOM="somservice"
SNV_HRN="hrnservice"

INSTALLED_CLI_MAJOR_VER=0
INSTALLED_CLI_RELEASE_VER=0
INSTALLED_HRN_MAJOR_VER=0
INSTALLED_HRN_RELEASE_VER=0

TARGET_PACKAGE_CLI=1
TARGET_PACKAGE_HRN=1



function do_usage() {
	echo "Usage:"
	echo "  $CMDNAME [option]"
	echo "    -i [snvcli | hrn]  install all system or optional package"
	echo "    -e [snvcli | hrn]  uninstall"
	echo "    -l                 show installed package detail"
	echo "       --help          show help message"
	echo "       --version       show installer version"
	exit 1
}

function do_wait() {
	## not use
	return 0
}

function do_check_installed_package() {
	do_putlog "check installed package."
	# << cli package >>
	rpm -q $CLI_PACKAGE_NAME >>$LOGFILE 2>>$LOGFILE
	if [ $? -eq 0 ] ; then
		## already installed
		rpm -qi $CLI_PACKAGE_NAME >>$LOGFILE 2>>$LOGFILE
		cur_ver=`rpm -q $CLI_PACKAGE_NAME | cut -d"-" -f2`
		INSTALLED_CLI_MAJOR_VER=`echo $cur_ver | cut -d"." -f2`
		INSTALLED_CLI_RELEASE_VER=`rpm -q $CLI_PACKAGE_NAME | cut -d"-" -f3`
		do_putlog "installed snvcli MAJOR[$INSTALLED_CLI_MAJOR_VER], RELEASE[$INSTALLED_CLI_RELEASE_VER]"
	fi
	# << hrn package >>
	rpm -q $HRN_PACKAGE_NAME >>$LOGFILE 2>>$LOGFILE
	if [ $? -eq 0 ] ; then
		## already installed
		rpm -qi $HRN_PACKAGE_NAME >>$LOGFILE 2>>$LOGFILE
		INSTALLED_HRN_MAJOR_VER=`rpm -q $HRN_PACKAGE_NAME | cut -d"-" -f2`
		INSTALLED_HRN_RELEASE_VER=`rpm -q $HRN_PACKAGE_NAME | cut -d"-" -f3`
		do_putlog "installed hrn MAJOR[$INSTALLED_HRN_MAJOR_VER], RELEASE[$INSTALLED_HRN_RELEASE_VER]"
	fi
}

function do_check_depend_package() {
	depflag=0
	do_putlog "check dependecy package."
	# << tcl >>
	rpm -q tcl >>$LOGFILE 2>>$LOGFILE
	if [ $? -ne 0 ] ; then
		depflag=`expr $depflag + 100`
	fi
	# << expect >>
	rpm -q expect >>$LOGFILE 2>>$LOGFILE
	if [ $? -ne 0 ] ; then
		depflag=`expr $depflag + 10`
	fi
	# << iconv >>
	rpm -q glibc >>$LOGFILE 2>>$LOGFILE
	if [ $? -ne 0 ] ; then
		depflag=`expr $depflag + 1`
	fi

	if [ $depflag -ne 0 ]; then
		echo "Failed in installation."
		if [ $depflag -ge 100 ]; then
			echo "Because a necessary component is not installed.(tcl)"
			depflag=`expr $depflag - 100`
		fi
		if [ $depflag -ge 10 ]; then
			echo "Because a necessary component is not installed.(expect)"
			depflag=`expr $depflag - 10`
		fi
		if [ $depflag -ge 1 ]; then
			echo "Because a necessary component is not installed.(glibc)"
		fi
		exit 1
	fi
}

function do_service_stop() {

	## hrnservice control is hrn's task.

	if [ $TARGET_PACKAGE_CLI -eq 1 ] ; then
		if [ $INSTALLED_CLI_MAJOR_VER -ne 0 ]; then
			# somservice stop wait.
			echo "Stop the $SNV_SOM"
			do_putlog "Stop the SOM Service."
			do_wait $SNV_SOM 10
			/sbin/service $SNV_SOM stop >>$LOGFILE 2>>$LOGFILE
		fi
	fi
}

function do_check_version() {
	EXP_VER=$1
	RES_VER=$2

	if [[ $EXP_VER =~ [0-9]+\.[0-9]+ ]]; then
		EXP_UPPER=`echo $EXP_VER | cut -d"." -f1`
		EXP_LOWER=`echo $EXP_VER | cut -d"." -f2`
	else
		EXP_UPPER=$EXP_VER
		EXP_LOWER=0
	fi

	if [[ $RES_VER =~ [0-9]+\.[0-9]+ ]]; then
		RES_UPPER=`echo $RES_VER | cut -d"." -f1`
		RES_LOWER=`echo $RES_VER | cut -d"." -f2`
	else
		RES_UPPER=$RES_VER
		RES_LOWER=0
	fi

	if [ $EXP_UPPER -eq $RES_UPPER ]; then
		if [ $EXP_LOWER -eq $RES_LOWER ]; then
			# 1.1 vs 1.1
			echo 0
		elif [ $EXP_LOWER -gt $RES_LOWER ]; then
			# 1.3 vs 1.2
			echo 1
		else
			# 1.2 vs 1.3
			echo 2
		fi
	elif [ $EXP_UPPER -gt $RES_UPPER ]; then
		# 3.1 vs 2.9
		echo 1
	else
		echo 2
	fi
}

function do_install_error() {
	echo -e 'The installation failed.\n(Please eject the CDROM.)'
	do_putlog "The installation failed."
	exit 1
}

function do_rollback_error() {
	## rollback install major number failed
	do_putlog "rollback install major number failed"
	echo "The setup failed. (Please eject the CDROM.)"
	echo "A later version of the product is already installed "
	echo "on your computer. "
	echo "The setup cannot continue."
	exit 1
}

function do_install() {
	if [ $INSTALLED_CLI_MAJOR_VER -eq 0 -a "$INSTALLED_HRN_MAJOR_VER" = "0" ]; then
		## new install
		do_putlog "new install process."
		if [ $TARGET_PACKAGE_CLI -eq 0 ]; then
			# Error message
			echo -e 'Failed in installation.\nA necessary component is not installed.'
			do_putlog "error. not select snvcli"
			exit 1
		fi
	else
		## update (or rollback) install
		do_putlog "check install version."
		if [ $INSTALLED_CLI_MAJOR_VER -eq $CLI_MAJOR_VER ]; then
			RESULT=`do_check_version ${INSTALLED_CLI_RELEASE_VER} ${CLI_RELEASE_VER}`
			case $RESULT in
				0)
					do_putlog "$CLI_PACKAGE_NAME is same version (SKIP)." ;
					TARGET_PACKAGE_CLI=0 ;;
				1)
					do_putlog "$CLI_PACKAGE_NAME is rollback install (OK)." ;
					TARGET_PACKAGE_CLI=`expr $TARGET_PACKAGE_CLI + 2000` ;;
				2)
					do_putlog "$CLI_PACKAGE_NAME is update install (OK)." ;
					TARGET_PACKAGE_CLI=`expr $TARGET_PACKAGE_CLI + 1000` ;;
				*)
					echo "error!?" ;;
			esac
		elif [ $INSTALLED_CLI_MAJOR_VER -lt $CLI_MAJOR_VER ]; then
			do_putlog "$CLI_PACKAGE_NAME is update install (OK)."
			TARGET_PACKAGE_CLI=`expr $TARGET_PACKAGE_CLI + 1000`
		elif [ $INSTALLED_CLI_MAJOR_VER -gt $CLI_MAJOR_VER ]; then
			do_putlog "$CLI_PACKAGE_NAME is rollback install (NG)."
			do_rollback_error
		fi

		if [ "$INSTALLED_HRN_MAJOR_VER" = "0" -a $TARGET_PACKAGE_HRN -eq 0 ]; then
			do_putlog "$HRN_PACKAGE_NAME is not install"
		else
			MAJRES=`do_check_version ${INSTALLED_HRN_MAJOR_VER} ${HRN_MAJOR_VER}`
			if [ $MAJRES -eq 0 ]; then
				RESULT=`do_check_version ${INSTALLED_HRN_RELEASE_VER} ${HRN_RELEASE_VER}`
				case $RESULT in
					0)
						do_putlog "$HRN_PACKAGE_NAME is same version (SKIP)." ;
						TARGET_PACKAGE_HRN=0 ;;
					1)
						do_putlog "$HRN_PACKAGE_NAME is rollback install (NG)." ;
						do_rollback_error ;;
					2)
						if [ $TARGET_PACKAGE_HRN -ne 0 ] ; then
							do_putlog "$HRN_PACKAGE_NAME is update install (OK)." ;
							TARGET_PACKAGE_HRN=`expr $TARGET_PACKAGE_HRN + 1000`  ;
						fi ;;
					*)
						echo "error!?" ;;
				esac
			elif [ $MAJRES -eq 1 ]; then
				if [ $TARGET_PACKAGE_HRN -ne 0 ]; then
					do_putlog "$HRN_PACKAGE_NAME is rollback install (NG)."
					do_rollback_error
				fi
			elif [ $MAJRES -eq 2 ]; then
				if [ "$INSTALLED_HRN_MAJOR_VER" = "0" -a $TARGET_PACKAGE_CLI -gt 0 ]; then
					do_putlog "$HRN_PACKAGE_NAME is not install"
					TARGET_PACKAGE_HRN=0
				else
					do_putlog "$HRN_PACKAGE_NAME is update install (OK)."
					TARGET_PACKAGE_HRN=`expr $TARGET_PACKAGE_HRN + 1000`
				fi
			else
				echo "error!?"
			fi
		fi
	fi

	if [ $TARGET_PACKAGE_CLI -eq 0 -a $TARGET_PACKAGE_HRN -eq 0 ]; then
		## already installed same version
		do_putlog "already installed same version."
		echo "This system already installed same version."
		exit 0
	fi

	## check rpm file exists.
	if [ ! -e "$RPMPATH/$CLI_PACKAGE_NAME-${CLI_VERSION}.${ARCH}.rpm" -a $TARGET_PACKAGE_CLI -eq 1 ]; then
		# Error message
		echo -e "Failed in installation.\nNot found $CLI_PACKAGE_NAME-${CLI_VERSION}.${ARCH}.rpm file."
		do_putlog "$CLI_PACKAGE_NAME package is nothing in installer."
		exit 1
	elif [ ! -e "$RPMPATH/$HRN_PACKAGE_NAME-${HRN_VERSION}.${ARCH}.rpm" -a $TARGET_PACKAGE_HRN -eq 1 ]; then
		# Error message
		echo -e "Failed in installation.\nNot found $HRN_PACKAGE_NAME-${HRN_VERSION}.${ARCH}.rpm file."
		do_putlog "$HRN_PACKAGE_NAME package is nothing in installer."
		exit 1
	fi

	## service stop
	do_service_stop

	if [ $TARGET_PACKAGE_CLI -ne 0 ]; then
		LEVEL=`expr $TARGET_PACKAGE_CLI - 1000`
		if [ $LEVEL -gt 1 ]; then
			VER=${OS_KIND}.${INSTALLED_CLI_MAJOR_VER}-${INSTALLED_CLI_RELEASE_VER}
			do_putlog "delete old package (${CLI_PACKAGE_NAME}-${VER})."
			rpm -e ${CLI_PACKAGE_NAME}-${VER} --nodeps >>$LOGFILE 2>>$LOGFILE
		fi
		do_putlog "install $RPMPATH/$CLI_PACKAGE_NAME-${CLI_VERSION}.${ARCH}.rpm"
		rpm -Uhv --nodeps $RPMPATH/$CLI_PACKAGE_NAME-${CLI_VERSION}.${ARCH}.rpm >>$LOGFILE 2>>$LOGFILE
		if [ $? -ne 0 ] ; then
			do_putlog "install error. try force install."
			rpm -ihv --nodeps --force $RPMPATH/$CLI_PACKAGE_NAME-${CLI_VERSION}.${ARCH}.rpm >>$LOGFILE 2>>$LOGFILE
			if [ $? -ne 0 ] ; then
				do_install_error 1
			fi
		fi
	fi

	if [ $TARGET_PACKAGE_HRN -ne 0 ]; then
		do_putlog "install $RPMPATH/$HRN_PACKAGE_NAME-${HRN_VERSION}.${ARCH}.rpm"
		rpm -Uhv --nodeps $RPMPATH/$HRN_PACKAGE_NAME-${HRN_VERSION}.${ARCH}.rpm >>$LOGFILE 2>>$LOGFILE
		if [ $? -ne 0 ] ; then
			do_putlog "install error. try force install."
			rpm -ihv --nodeps --force $RPMPATH/$HRN_PACKAGE_NAME-${HRN_VERSION}.${ARCH}.rpm >>$LOGFILE 2>>$LOGFILE
			if [ $? -ne 0 ] ; then
				do_install_error 1
			fi
		fi
	fi

	if [ $TARGET_PACKAGE_CLI -eq 1 ]; then
		echo "Start the $SNV_SOM"
		do_putlog "Start the SOM Service."
		/sbin/service $SNV_SOM start >>$LOGFILE 2>>$LOGFILE
	fi	
	echo -e 'The installation was successful.\n(Please eject the CDROM.)'
	do_putlog "The installation was successful."
}


function do_uninstall() {
	if [ $INSTALLED_CLI_MAJOR_VER -eq 0 -a "$INSTALLED_HRN_MAJOR_VER" = "0" ]; then
		#not installed
		echo -e 'The uninstallation failed.\n(Please eject the CDROM.)'
		do_putlog "snvcli and hrn not found in this system."
		exit 1
	fi
	if [ $TARGET_PACKAGE_CLI -eq 0 -a $TARGET_PACKAGE_HRN -eq 1 -a "$INSTALLED_HRN_MAJOR_VER" = "0" ] ; then
		# not installed
		echo -e 'The uninstallation failed.\n(Please eject the CDROM.)'
		do_putlog "hrn not found in this system."
		exit 1
	fi
	if [ $TARGET_PACKAGE_CLI -eq 1 -a $TARGET_PACKAGE_HRN -eq 0 -a "$INSTALLED_HRN_MAJOR_VER" != "0" ] ; then
		# fail uninstall snvcli only
		echo -e 'The uninstallation failed.\n(Please eject the CDROM.)'
		do_putlog "can't uninstall only snvcli. hrn is installed."
		exit 1
	fi
	
	do_service_stop

	if [ $TARGET_PACKAGE_HRN -eq 1 -a "$INSTALLED_HRN_MAJOR_VER" != "0" ] ; then
		rpm -e --allmatches $HRN_PACKAGE_NAME >>$LOGFILE 2>>$LOGFILE
		if [ $? -ne 0 ] ; then
			echo -e 'The uninstallation failed.\n(Please eject the CDROM.)'
			do_putlog "hrn uninstallation failed."
			exit 1
		fi
	fi
	if [ $TARGET_PACKAGE_CLI -eq 1 -a $INSTALLED_CLI_MAJOR_VER -ne 0 ] ; then
		rpm -e --allmatches $CLI_PACKAGE_NAME >>$LOGFILE 2>>$LOGFILE
		if [ $? -ne 0 ] ; then
			echo -e 'The uninstallation failed.\n(Please eject the CDROM.)'
			do_putlog "snvcli uninstallation failed."
			exit 1
		fi
		
		rm -rf $SNVPATH
	fi

	echo -e 'The uninstallation was successful.\n(Please eject the CDROM.)'
	do_putlog "The uninstallation was successful."
}

function do_installer_ver() {
	echo "$CMDNAME Version = $VERSION"
}

function do_showinfo() {
	do_installer_ver
	rpm -q $CLI_PACKAGE_NAME >/dev/null 2>&1
	if [ $? -eq 0 ] ; then
		echo " "
		rpm -qi $CLI_PACKAGE_NAME
	fi
	rpm -q $HRN_PACKAGE_NAME >/dev/null 2>&1
	if [ $? -eq 0 ] ; then
		echo " "
		rpm -qi $HRN_PACKAGE_NAME
	fi
}

function do_putlog() {
	DATETIME=`date`
	echo "$DATETIME $1" >>$LOGFILE 2>/dev/null
}

function do_newlog() {
	rm -f $LOGFILE 2>&1
	mkdir -p $LOGDIR >/dev/null 2>&1
	echo -n "" >$LOGFILE 2>&1
}

function do_checksupportos() {
	if [ ! -e /etc/vma-release ]; then
		echo -e 'The setup failed. (Please eject the CDROM.)\nThis OS is not supported.'
		do_putlog "This OS is not supported. (name)"
		uname -s >>$LOGFILE
		exit 1 >/dev/null 2>&1
	fi
}

function do_checkdiskfreespace() {
	TARGET_DIR=/opt
	if expr "`df -m ${TARGET_DIR} | tail -n 1`" : " " > /dev/null
	then
		FREEMB=`df -m ${TARGET_DIR} | tail -n 1 | awk '{ print $3 }'`
	else
		FREEMB=`df -m ${TARGET_DIR} | tail -n 1 | awk '{ print $4 }'`
	fi
	
	if [ ${FREEMB} -lt 20 ]; then
        echo -e "The installation failed.Because there is not enough space for the installation."
        echo -e "Please confirm the free space to install"
		do_putlog "Disk free space is too small. (${FREEMB}MB)"
		exit 1 >/dev/null 2>&1
	fi
}

function do_multi_exec_check() {
	sync
	ps aux | grep [s]nvsetup_vm > TMPPROCS
	count=`cat TMPPROCS | wc -l`
	EXEC_COUNT=99

	## setup.sh already executed?
	if [ $count -eq 2 ]; then
		cat TMPPROCS | head -n 1 | grep "sudo" > /dev/null
		## process executed with sudo comannd -> OK
		if [ $? -eq 0 ]; then
			EXEC_COUNT=0
		fi
	elif [ $count -gt 1 ]; then
		## error: find multiple pids
		do_putlog "`cat TMPPROCS`"
	else
		EXEC_COUNT=0
	fi
	rm -f TMPPROCS
}

USERID=`id | sed 's/uid=\([0-9]*\)(.*/\1/'`
if [ $USERID -ne 0 ];then
	echo "Permission denied."
    exit 1 >/dev/null 2>&1
fi 

CMD=$1
if [ "$CMD" = "-i" ];then
	do_multi_exec_check
	if [ $EXEC_COUNT -ne 0 ]
	then
		echo -e 'The installation failed. (There are multiple installation processes.)'
		#do_putlog "Error Multiple starts"
		exit 1 >/dev/null 2>&1
	fi
	do_newlog
	do_putlog "start install."
	if [ "$2" = "snvcli" ]; then
		do_putlog "target $2."
		TARGET_PACKAGE_CLI=1
		TARGET_PACKAGE_HRN=0
		do_checksupportos
	elif [ "$2" = "hrn" ]; then
		do_putlog "target $2."
		TARGET_PACKAGE_CLI=0
		TARGET_PACKAGE_HRN=1
		do_checksupportos
	elif [ "$2" != "" ]; then
		do_putlog "unknown paramater $2"
		do_usage
	else
		TARGET_PACKAGE_CLI=1
		TARGET_PACKAGE_HRN=1
		do_checksupportos
	fi
	do_check_depend_package
	do_check_installed_package
	do_checkdiskfreespace
	do_install
elif [ "$CMD" = "-e" ];then
	do_multi_exec_check
	if [ $EXEC_COUNT -ne 0 ]
	then
		echo -e 'The installation failed. (There are multiple installation processes.)'
		#do_putlog "Error Multiple starts"
		exit 1 >/dev/null 2>&1
	fi
	do_newlog
	do_putlog "start uninstall."
	if [ "$2" = "snvcli" ]; then
		do_putlog "target $2."
		TARGET_PACKAGE_CLI=1
		TARGET_PACKAGE_HRN=0
	elif [ "$2" = "hrn" ]; then
		do_putlog "target $2."
		TARGET_PACKAGE_CLI=0
		TARGET_PACKAGE_HRN=1
	elif [ "$2" != "" ]; then
		do_putlog "unknown paramater $2"
		do_usage
	fi
	do_checksupportos
	do_check_installed_package
	do_uninstall
elif [ "$CMD" = "-l" ];then
	do_showinfo
elif [ "$CMD" = "--version" ];then
	do_installer_ver
else
	do_usage
fi

